/******************************************************************************
 *
 * Licensed Materials - Property of IBM
 *
 * Source File Name = UniqueUserID.java
 *
 * (C) COPYRIGHT IBM Corp. 2005
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *****************************************************************************/


import FESI.jslib.*;
import com.ibm.itim.dataservices.model.*;
import com.ibm.itim.dataservices.model.domain.*;
import com.ibm.itim.fesiextensions.*;


/**
 * Identity Policy JavaScript extension for provisioning policy.
 *
 * Functions provided : getUniqueUserID
 */


public class UniqueUserID implements JSExtension {


	private static String Account_Owner = "owner";
	private static String ACCOUNT_ID = "eruid";
	private static CompoundDN searchContext = null;
	public static boolean DEBUG = false; //Set to false to avoid any debug output from extension.
	private static String SERVICE_ID = "erservice";



	/**
	 * Initializes the extension.
	 * @param go JSGlobalObject for the interpreter thread.
	 * @exception JSException Thrown if unable to initialize the extension.
	 */

	public void initializeExtension(JSGlobalObject go) throws JSException {

		if(DEBUG) System.out.println("UniqueUserID..Registering the extension function.");

	  /**
		* getUniqueUserID returns empty string, if no account is found for person. Otherwise returns existing userID.
		* @param1 Person's erGlobalid.
		* @param2 Service's erGlobalid.
		*/
		go.setMember("getUniqueUserID", new GetUniqueUserID());

	}


	private class GetUniqueUserID extends JSFunctionAdapter
	{
		public Object doCall(JSObject thisObject, Object [] args) throws JSException {

			if(args.length == 0)
				throw new JSException("No arguments provided.Minimum one argument required.");

			String erGlobalid = (String) args[0];
			String serviceId = "";
			SearchResults sr = null;

			if(erGlobalid == null)
				throw new JSException("No arguments provided.Minimum one argument required.");

			if(args.length > 1) {
				serviceId = (String) args[1];
				serviceId = serviceId.toLowerCase();
			}

			PersonEntity person =
            (PersonEntity)thisObject.getGlobalObject().getMember(IdentityPolicyRuntimeContext.FESI_CONTEXT_ITEM_PERSON);
            DistinguishedName tenantDN = person.getDirectoryObject().getTenantDN();
            searchContext = new CompoundDN(tenantDN);
			try{

				SearchParameters searchParams = new SearchParameters();
				searchParams.setScope(SearchParameters.SUBTREE_SCOPE);
				AccountSearch accountSearch = new AccountSearch();

				String ownerFilter = "(" + "owner =" + "*" + erGlobalid +"*" +")";
				String serviceFilter = "";

				sr  = accountSearch.searchByFilter(searchContext,null,ownerFilter,searchParams);

				//Check, whether any account is created for person or not.
				// If any account is created for person, return existing user ID.

				if(sr != null){

					SearchResultsIterator it = sr.iterator();
					while(it.hasNext()){


						DirectoryObject dirObj = ((AccountEntity) it.next()).getDirectoryObject();
						String eruid = (dirObj.getAttribute(ACCOUNT_ID)).getString();
						serviceFilter = "(" + ACCOUNT_ID + "=" + eruid + ")";

						if(!serviceId.equals("")){

							serviceFilter = "(&" + serviceFilter + "(" + SERVICE_ID + "=*" + serviceId + "* ))";
							sr.close();
							sr  = accountSearch.searchByFilter(searchContext,null,serviceFilter,searchParams);
							if(sr != null){

								it = sr.iterator();
								if(it.hasNext())
								{
									sr.close();
									return "";
								}
								sr.close();
							}
						}
						return eruid;
					}

					return "";
				}

				else
					return "";
			}
			catch(Exception e) {
				if(DEBUG) System.out.println("UniqueUserID..Checking if eruid exists - exception caught");
				return new Integer(-1);

			}finally {
				try{
					if(sr != null && !sr.isClosed())
						sr.close();
				}catch(Exception e) {
						if(DEBUG) System.out.println("UniqueUserID..Checking if eruid exists - exception caught");
						return new Integer(-1);
					}
			}

		}

	}
}